"""Banner Controlpanel API"""

from zope.interface import Interface
from zope.component import adapter
from plone.restapi.controlpanels import RegistryConfigletPanel
from eea.banner.interfaces import IBannerSettings
from eea.banner.interfaces import IEeaBannerLayer


@adapter(Interface, IEeaBannerLayer)
class BannerControlpanel(RegistryConfigletPanel):
    """Banner Control Panel"""

    schema = IBannerSettings
    schema_prefix = None
    configlet_id = "banner"
    configlet_category_id = "Products"
    title = "Banner Settings"
    group = "Products"



"""RestAPI enpoint @banner GET"""

import os
import json
from contextlib import closing
from six.moves import urllib
from plone import api
from plone.restapi.services import Service
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

# from eea.cache import cache
from eea.banner.interfaces import IBannerSettings, IEeaBannerLayer

TIMEOUT = 15
RANCHER_METADATA = "http://rancher-metadata/latest"
MEMCACHE_AGE = 300


def isTrue(value):
    """Evaluate True"""
    if isinstance(value, str):
        return value.lower() in ("true", "1", "t", "on", "yes", "y")
    if isinstance(value, bool):
        return value
    return False


@implementer(IPublishTraverse)
class BannerGet(Service):
    """Banner GET"""

    def get_rancher_metadata(self, url):
        """Returns Rancher metadata API"""
        try:
            req = urllib.request.Request(url, headers={"Accept": "application/json"})
            with closing(urllib.request.urlopen(req, timeout=TIMEOUT)) as conn:
                result = json.loads(conn.read())
        except Exception:
            result = []
        return result

    def get_stacks(self):
        """Returns all Rancher stacks from the current environment"""
        url = "%s/stacks" % RANCHER_METADATA
        return self.get_rancher_metadata(url)

    # @cache(lambda *args: "rancher-status", lifetime=MEMCACHE_AGE)
    def get_stacks_status(self, stacks):
        """Returns status of required stacks"""
        status = None
        rancher_stacks = self.get_stacks()
        for stack in rancher_stacks:
            if stack.get("system") or stack.get("name") not in stacks:
                continue
            services = stack.get("services", [])
            if not services:
                continue
            for service in services:
                if not status and service.get("state") != "active":
                    status = service.get("state")
                    break
        return status

    def reply(self):
        """Reply"""

        if not IEeaBannerLayer.providedBy(self.request):
            return {
                "static_banner": {"enabled": False},
                "dynamic_banner": {"enabled": False},
            }
        development = self.request.form.get("development")

        dynamic_banner_env = "DYNAMIC_BANNER_ENABLED_" + getSite().getId()
        dynamic_banner_enabled = isTrue(
            os.getenv(dynamic_banner_env, False)
        ) or api.portal.get_registry_record(
            "dynamic_banner_enabled",
            interface=IBannerSettings,
            default=False,
        )

        static_banner_env = "STATIC_BANNER_ENABLED_" + getSite().getId()
        static_banner_enabled = isTrue(
            os.getenv(static_banner_env, False)
        ) or api.portal.get_registry_record(
            "static_banner_enabled",
            interface=IBannerSettings,
            default=False,
        )

        return {
            "static_banner": {
                "enabled": static_banner_enabled,
                "visible_to_all": api.portal.get_registry_record(
                    "static_banner_visible_to_all",
                    interface=IBannerSettings,
                    default=False,
                ),
                "type": api.portal.get_registry_record(
                    "static_banner_type", interface=IBannerSettings, default=""
                ),
                "title": api.portal.get_registry_record(
                    "static_banner_title",
                    interface=IBannerSettings,
                    default="",
                ),
                "message": api.portal.get_registry_record(
                    "static_banner_message",
                    interface=IBannerSettings,
                    default="",
                ),
            },
            "dynamic_banner": {
                "enabled": dynamic_banner_enabled,
                "visible_to_all": api.portal.get_registry_record(
                    "dynamic_banner_visible_to_all",
                    interface=IBannerSettings,
                    default=False,
                ),
                "title": api.portal.get_registry_record(
                    "dynamic_banner_title",
                    interface=IBannerSettings,
                    default="",
                ),
                "message": api.portal.get_registry_record(
                    "dynamic_banner_message",
                    interface=IBannerSettings,
                    default="",
                ),
                "rancher_stacks_status": None
                if development or not dynamic_banner_enabled
                else self.get_stacks_status(
                    api.portal.get_registry_record(
                        "rancher_stacks", interface=IBannerSettings, default=[]
                    )
                    or []
                ),
            },
        }



"""Doc tests"""

import doctest
import unittest
from eea.banner.tests.base import FUNCTIONAL_TESTING
from plone.testing import layered

OPTIONFLAGS = (
    doctest.REPORT_ONLY_FIRST_FAILURE | doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
)


def test_suite():
    """Suite"""
    suite = unittest.TestSuite()
    suite.addTests(
        [
            layered(
                doctest.DocFileSuite(
                    "README.txt", optionflags=OPTIONFLAGS, package="eea.banner"
                ),
                layer=FUNCTIONAL_TESTING,
            ),
        ]
    )

    return suite


if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")



"""Base test cases"""

from Products.CMFPlone import setuphandlers
from plone.testing import z2
from plone.app.testing import TEST_USER_ID
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import setRoles


class EEAFixture(PloneSandboxLayer):
    """EEA Testing Policy"""

    def setUpZope(self, app, configurationContext):
        """Setup Zope"""
        import eea.banner

        self.loadZCML(package=eea.banner)
        z2.installProduct(app, "eea.banner")

    def setUpPloneSite(self, portal):
        """Setup Plone"""
        applyProfile(portal, "eea.banner:default")

        # Default workflow
        wftool = portal["portal_workflow"]
        wftool.setDefaultChain("simple_publication_workflow")

        # Login as manager
        setRoles(portal, TEST_USER_ID, ["Manager"])

        # Add default Plone content
        try:
            applyProfile(portal, "plone.app.contenttypes:plone-content")
        except KeyError:
            # BBB Plone 4
            setuphandlers.setupPortalContent(portal)

        # Create testing environment
        portal.invokeFactory("Folder", "sandbox", title="Sandbox")

    def tearDownZope(self, app):
        """Uninstall Zope"""
        z2.uninstallProduct(app, "eea.banner")


EEAFIXTURE = EEAFixture()
FUNCTIONAL_TESTING = FunctionalTesting(bases=(EEAFIXTURE,), name="EEAbanner:Functional")



"""Control panel module"""

from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm

from eea.banner.interfaces import IBannerSettings


class BannerRegistryEditForm(RegistryEditForm):
    """Banner Registry Edit Form"""

    schema = IBannerSettings
    id = "banner"
    label = "Banner Settings"


class BannerControlPanelFormWrapper(ControlPanelFormWrapper):
    """Banner Control Panel Form Wrapper"""

    form = BannerRegistryEditForm



"""Module where all interfaces, events and exceptions live."""

from zope import schema
from zope.schema.vocabulary import SimpleVocabulary
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from eea.banner import EEAMessageFactory as _


class IEeaBannerLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IBannerSettings(Interface):
    """Client settings for EEA Banner."""

    static_banner_enabled = schema.Bool(
        title=_("Enable / disable static banner"),
        description=_(
            "If STATIC_BANNER_ENABLED environment variable is 'on' "
            "this flag has no effect"
        ),
        default=True,
        required=False,
    )

    static_banner_visible_to_all = schema.Bool(
        title=_("Show static banner to anonymous users?"),
        default=True,
        required=False,
    )

    static_banner_type = schema.Choice(
        title=_("Static banner type"),
        default="warning",
        vocabulary=SimpleVocabulary.fromValues(["success", "warning", "error"]),
    )

    static_banner_title = schema.TextLine(
        title=_("Static banner title"),
        default="This is a demo/test instance",
        required=False,
    )

    static_banner_message = schema.Text(
        title=_("Static banner message"),
        required=False,
        default=(
            "Do not use it for operational purposes. "
            "All changes will be regularly overwritten"
        ),
    )

    dynamic_banner_enabled = schema.Bool(
        title=_("Enable / disable dynamic banner"),
        default=True,
        required=False,
        description=_(
            "It will appear only if status of at least one stack "
            "is not 'active'. "
            "If STATIC_BANNER_ENABLED environment variable is 'on' "
            "this flag has no effect"
        ),
    )

    dynamic_banner_visible_to_all = schema.Bool(
        title=_("Show dynamic banner to anonymous users?"),
        default=True,
        required=False,
    )

    rancher_stacks = schema.List(
        title=_("Rancher stacks to monitor"),
        default=[],
        required=False,
        value_type=schema.TextLine(),
    )

    dynamic_banner_title = schema.TextLine(
        title=_("Dynamic banner title"),
        default="Web admins says:",
        required=False,
    )

    dynamic_banner_message = schema.Text(
        title=_("Dynamic banner message"),
        required=False,
        default="The system is {}",
        description=(
            "Add suffix/prefix to rancher stacks status message. "
            "Use {} for rancher stacks status placeholder"
        ),
    )



"""Main product initializer"""

from zope.i18nmessageid.message import MessageFactory

EEAMessageFactory = MessageFactory("eea")


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



"""eea.banner Installer"""

import os
from os.path import join
from setuptools import setup, find_packages

NAME = "eea.banner"
PATH = NAME.split(".") + ["version.txt"]
VERSION = open(join(*PATH)).read().strip()

setup(
    name=NAME,
    version=VERSION,
    description="volto-banner backend support",
    long_description_content_type="text/x-rst",
    long_description=(
        open("README.rst").read()
        + "\n"
        + open(os.path.join("docs", "HISTORY.txt")).read()
    ),
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="EEA Add-ons Plone Zope",
    author="European Environment Agency: IDM2 A-Team",
    author_email="eea-edw-a-team-alerts@googlegroups.com",
    url="https://github.com/eea/eea.banner",
    license="GPL version 2",
    packages=find_packages(exclude=["ez_setup"]),
    namespace_packages=["eea"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "setuptools",
        # -*- Extra requirements: -*-
    ],
    extras_require={
        "test": [
            "plone.app.testing",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
